// 
// The Waveset SPML General License 
// 
// Version 0.1, April 2003
// Copyright (C) 2003 Waveset Technologies, Inc..
// 6034 West Courtyard Drive, Suite 210, Austin, Texas 78730
// All rights reserved.
// 
// TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
// 
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions, and the disclaimers in Sections 6
//    and 7 below.
// 
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the disclaimers in Sections 6
//    and 7 below in the documentation and/or other materials provided
//    with the distribution.
// 
// 3. The end-user documentation included with the redistribution, if
//    any, must include the following acknowledgment:
// 
//    "This product includes software developed by 
//     Waveset Technologies, Inc. (www.waveset.com)."
// 
//    Alternately, this acknowledgment may appear in the software itself, if
//    and wherever such third-party acknowledgments normally appear.
//  
// 4. The names "Waveset" and "Waveset Technologies, Inc." must not be
//    used to endorse or promote products derived from this software
//    without the prior written permission of Waveset. For written
//    permission, please contact www.waveset.com.
//  
// 5. Products derived from this software may not be called "Waveset",
//    nor may "Waveset" appear in their name, without the prior written
//    permission of Waveset.
//  
// 6. NO WARRANTY 
// 
//    THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED
//    OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//    DISCLAIMED.
// 
// 7.  LIMITATION OF LIABILITY
// 
//    IN NO EVENT SHALL THE WAVESET OR ITS LICENSORS BE LIABLE FOR ANY
//    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
//    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
//    GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
//    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
//    IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
//    OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
//    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 
// End of Terms and Conditions
// ====================================================================
//
// Component Name: org.openspml.message.ObjectClassDefinitionReference
//
// Author(s): Jeff Larson
//
// Description:
//
// Memory representation for an <objectClassDefinitionReference> element.
//
//

package org.openspml.message;

import java.util.*;
import org.openspml.util.*;

/**
 * A class representing an <code>attributeDefinitionReference</code> element.
 */
public class ObjectClassDefinitionReference {

    //////////////////////////////////////////////////////////////////////
    //
    // Fields
    //
    // In theory, have to allow arbitrary attributes and elements.
    //
    //////////////////////////////////////////////////////////////////////

    static final String ELEMENT = "objectClassDefinitionReference";

    /**
     * Parent schema for reference resolution.
     */
    Schema _schema;

    /**
     * The name of the referenced class.
     */
    String _name;

    /**
     * Identifier of the containing schema.
     */
    SchemaIdentifier _schemaIdentifier;

    /**
     * Identifier of the schema provider.
     */
    ProviderIdentifier _providerIdentifier;

    /**
     * Transient resolved definition.
     */
    ObjectClassDefinition _definition;

    //////////////////////////////////////////////////////////////////////
    //
    // Constructors
    //
    //////////////////////////////////////////////////////////////////////

    public ObjectClassDefinitionReference() {
    }

    ObjectClassDefinitionReference(XmlElement e) {
	parseXml(e);
    }

    void setSchema(Schema s) {
        _schema = s;
    }

    public void setName(String s) {
	_name = s;
    }

    public void setProviderIdentifier(ProviderIdentifier id) {
        _providerIdentifier = id;
    }

    public void setProviderIdentifier(String id) {
        _providerIdentifier = new ProviderIdentifier(id);
    }

    public void setSchemaIdentifier(SchemaIdentifier id) {
        _schemaIdentifier = id;
    }

    public void setSchemaIdentifier(String s) {
        _schemaIdentifier = new SchemaIdentifier(s);
    }

    //////////////////////////////////////////////////////////////////////
    //
    // Accessors
    //
    //////////////////////////////////////////////////////////////////////
    
    public String getName() {
	return _name;
    }

    public ProviderIdentifier getProviderIdentifier() {
	return _providerIdentifier;
    }

    public String getProviderIdentifierString() {
	return (_providerIdentifier != null) ? 
            _providerIdentifier.getId() : null;
    }

    public SchemaIdentifier getSchemaIdentifier() {
	return _schemaIdentifier;
    }

    public String getSchemaIdentifierString() {
	return (_schemaIdentifier != null) ? 
            _schemaIdentifier.getId() : null;
    }

    public ObjectClassDefinition getDefinition() {
        ObjectClassDefinition def = null;
        if (_definition == null && _schema != null && _name != null) {
            // ignore provider/schema identifiers for now, assume local
            _definition = _schema.getObjectClassDefinition(_name);
        }
        return _definition;
    }

    //////////////////////////////////////////////////////////////////////
    //
    // XML
    //
    //////////////////////////////////////////////////////////////////////

    void toXml(SpmlBuffer b) {

        b.addOpenStartTag(ELEMENT);
	b.addAttribute("name", _name);

        if (_providerIdentifier == null && _schemaIdentifier == null) 
            b.append("/>\n");
        else {
            b.closeStartTag();
            b.incIndent();

            // spec uses <xsd:sequence> so they must be
            // in this order

            if (_providerIdentifier != null)
                _providerIdentifier.toXml(b);

            if (_schemaIdentifier != null)
                _schemaIdentifier.toXml(b);

            b.decIndent();
            b.addEndTag(ELEMENT);
        }
    }

    void parseXml(XmlElement e) {

	// in theory, need to recognize and maintain 
	// other arbitrary elements

	_name = e.getAttribute("name");

	for (XmlElement child = e.getChildElement() ; 
	     child != null ; child = child.next()) {

	    String tag = child.getLocalName();

	    if (tag.equals(ProviderIdentifier.ELEMENT))
                _providerIdentifier = new ProviderIdentifier(child);

	    else if (tag.equals(SchemaIdentifier.ELEMENT))
                _schemaIdentifier = new SchemaIdentifier(child);

	    // in theory, need to recognize and maintain 
	    // other arbitrary elements

        }
    }

}
